paraslash 0.7.3
[paraslash.git] / recv.h
diff --git a/recv.h b/recv.h
index cc44a602a9f3ca2f63c68c8c984fb83c87f82703..391395b241d67abf12eb70996a10ee7950519087 100644 (file)
--- a/recv.h
+++ b/recv.h
@@ -1,8 +1,4 @@
-/*
- * Copyright (C) 2005-2013 Andre Noll <maan@systemlinux.org>
- *
- * Licensed under the GPL v2. For licencing details see COPYING.
- */
+/* Copyright (C) 2005 Andre Noll <maan@tuebingen.mpg.de>, see file COPYING. */
 
 /** \file recv.h Receiver-related structures and exported symbols of recv_common.c. */
 
  */
 struct receiver_node {
        /** Points to the corresponding receiver. */
-       struct receiver *receiver;
+       const struct receiver *receiver;
        /** Receiver-specific data. */
        void *private_data;
-       /** Pointer to the configuration data for this instance. */
-       void *conf;
+       /** The parsed command line options for this instance. */
+       struct lls_parse_result *lpr;
        /** The task associated with this instance. */
-       struct task task;
+       struct task *task;
        /** The receiver node is always the root of the buffer tree. */
        struct btr_node *btrn;
        /** Each receiver node maintains a buffer pool for the received data. */
@@ -25,11 +21,11 @@ struct receiver_node {
        /**
         * The file descriptor to receive the stream.
         *
* The pre_select function of the receiver adds this file descriptor to
         * the set of file descriptors which are watched for readability or
         * writability, depending on the state of the connection (if any).
         *
* If \a fd is readable, the post_select function of the receiver reads
         * data from this fd into the buffer pool area of \a btrp.
         *
         * \sa \ref receiver.
@@ -38,50 +34,26 @@ struct receiver_node {
 };
 
 /**
- * Describes one supported paraslash receiver.
+ * Describes a possible data source for audio streams.
+ *
+ * A paraslash receiver is a modular piece of software which is capable of
+ * receiving an audio data stream from a data source. Received audio data is
+ * fed to consumers through the buffer tree mechanism.
+ *
+ * This structure contains the methods which have to be implemented by each
+ * receiver.
  *
- * \sa http_recv.c, udp_recv.c
+ * \sa \ref http_recv.c, \ref udp_recv.c, \ref dccp_recv.c, \ref afh_recv.c,
+ * struct \ref receiver_node, struct \ref filter, struct \ref writer, struct
+ * \ref sched.
  */
 struct receiver {
-       /**
-        * The name of the receiver.
-        */
-       const char *name;
-       /**
-        * The receiver init function.
-        *
-        * It must fill in all other function pointers and is assumed to succeed.
-        *
-        * \sa http_recv_init udp_recv_init.
-        */
-       void (*init)(struct receiver *r);
-       /**
-        * The command line parser of the receiver.
-        *
-        * It should check whether the command line options given by \a argc and \a
-        * argv are valid.  On success, it should return a pointer to the
-        * receiver-specific configuration data determined by \a argc and \a argv.
-        * Note that this might be called more than once with different values of
-        * \a argc and \a argv.
-        */
-       void *(*parse_config)(int argc, char **argv);
-       /**
-        * Deallocate the configuration structure of a receiver node.
-        *
-        * This calls the receiver-specific cleanup function generated by
-        * gengetopt.
-        */
-       void (*free_config)(void *conf);
        /**
         * Open one instance of the receiver.
         *
-        * This should allocate the output buffer of \a rn. and may also
-        * perform any other work necessary for retrieving the stream according
-        * to the configuration stored in the \a conf member of \a rn which is
-        * guaranteed to point to valid configuration data (as previously
-        * obtained from the config parser).
-        *
-        * \sa receiver_node::conf, receiver_node::buf.
+        * This should allocate the output buffer of the given receiver node
+        * and prepare it for retrieving the audio stream according to the
+        * configuration stored in rn->lpr.
         */
        int (*open)(struct receiver_node *rn);
        /**
@@ -90,69 +62,38 @@ struct receiver {
         * It should free all resources associated with given receiver node
         * that were allocated during the corresponding open call.
         *
-        * \sa receiver_node.
+        * \sa \ref receiver_node.
         */
        void (*close)(struct receiver_node *rn);
-       /**
-        * Add file descriptors to fd_sets and compute timeout for select(2).
-        *
-        * The pre_select function gets called from the driving application
-        * before entering its select loop. The receiver may use this hook to
-        * add any file descriptors to the sets of file descriptors given by \a
-        * s.
-        *
-        * \sa select(2), time.c struct task, struct sched.
-        */
-       void (*pre_select)(struct sched *s, struct task *t);
-       /**
-        * Evaluate the result from select().
-        *
-        * This hook gets called after the call to select(). It should check
-        * all file descriptors which were added to any of the fd sets during
-        * the previous call to pre_select. According to the result, it may
-        * then use any non-blocking I/O to establish a connection or to
-        * receive the audio data.
-        *
-        * \sa select(2), struct receiver.
-        */
-       int (*post_select)(struct sched *s, struct task *t);
-
-       /** The two help texts of this receiver. */
-       struct ggo_help help;
+       /** Ask the scheduler to monitor receive fds. */
+       void (*pre_monitor)(struct sched *s, void *context);
+       /** Receive data and make it available to consumers. */
+       int (*post_monitor)(struct sched *s, void *context);
        /**
         * Answer a buffer tree query.
         *
-        * This optional function pointer is used for inter node communications
+        * This optional function pointer allows for inter node communication
         * of the buffer tree nodes. See \ref btr_command_handler for details.
         */
        btr_command_handler execute;
 };
 
-/** Define an array of all available receivers. */
-#define DEFINE_RECEIVER_ARRAY struct receiver receivers[] = { \
-       HTTP_RECEIVER \
-       DCCP_RECEIVER \
-       UDP_RECEIVER \
-       AFH_RECEIVER \
-       {.name = NULL}};
+#define RECV_CMD(_num) (lls_cmd(_num, recv_cmd_suite))
 
-/** Iterate over all available receivers. */
-#define FOR_EACH_RECEIVER(i) for (i = 0; receivers[i].name; i++)
+#define RECV_CMD_OPT_RESULT(_recv, _opt, _lpr) \
+       (lls_opt_result(LSG_RECV_CMD_ ## _recv ## _OPT_ ## _opt, _lpr))
+#define RECV_CMD_OPT_GIVEN(_recv, _opt, _lpr) \
+       (lls_opt_given(RECV_CMD_OPT_RESULT(_recv, _opt, _lpr)))
+#define RECV_CMD_OPT_STRING_VAL(_recv, _opt, _lpr) \
+       (lls_string_val(0, RECV_CMD_OPT_RESULT(_recv, _opt, _lpr)))
+#define RECV_CMD_OPT_UINT32_VAL(_recv, _opt, _lpr) \
+       (lls_uint32_val(0, RECV_CMD_OPT_RESULT(_recv, _opt, _lpr)))
+#define RECV_CMD_OPT_INT32_VAL(_recv, _opt, _lpr) \
+       (lls_int32_val(0, RECV_CMD_OPT_RESULT(_recv, _opt, _lpr)))
 
-void recv_init(void);
-void *check_receiver_arg(char *ra, int *receiver_num);
-void print_receiver_helps(int detailed);
-int generic_recv_pre_select(struct sched *s, struct task *t);
-
-/** \cond receiver */
-extern void http_recv_init(struct receiver *r);
-#define HTTP_RECEIVER {.name = "http", .init = http_recv_init},
-extern void dccp_recv_init(struct receiver *r);
-#define DCCP_RECEIVER {.name = "dccp", .init = dccp_recv_init},
-extern void udp_recv_init(struct receiver *r);
-#define UDP_RECEIVER {.name = "udp", .init = udp_recv_init},
-#define AFH_RECEIVER /* not active by default */
-
-extern struct receiver receivers[];
-/** \endcond receiver */
+/** Iterate over all available receivers. */
+#define FOR_EACH_RECEIVER(i) for (i = 1; lls_cmd(i, recv_cmd_suite); i++)
 
+int check_receiver_arg(const char *ra, struct lls_parse_result **lprp);
+void print_receiver_helps(bool detailed);
+int generic_recv_pre_monitor(struct sched *s, struct receiver_node *rn);
@@ -25,11 +21,11 @@ struct receiver_node {
         * The file descriptor to receive the stream.
         *
-        * The pre_select function of the receiver adds this file descriptor to
+        * The pre_monitor function of the receiver adds this file descriptor to
         * the set of file descriptors which are watched for readability or
         * writability, depending on the state of the connection (if any).
@@ -25,11 +21,11 @@ struct receiver_node {
         * writability, depending on the state of the connection (if any).
         *
-        * If \a fd is readable, the post_select function of the receiver reads
+        * If \a fd is readable, the post_monitor function of the receiver reads
         * data from this fd into the buffer pool area of \a btrp.
         *